import re
from distutils.version import StrictVersion

import django.conf.urls as urls
from django.conf import settings
from django.contrib.sites.models import Site
from django.urls import resolvers
from django.utils import deprecation

from maintenancemode.models import Maintenance
from maintenancemode.utils.settings import (
    DJANGO_VERSION, MAINTENANCE_ADMIN_IGNORED_URLS, MAINTENANCE_BLOCK_STAFF)

urls.handler503 = 'maintenancemode.views.defaults.temporary_unavailable'
urls.__all__.append('handler503')

_base = deprecation.MiddlewareMixin if DJANGO_VERSION >= StrictVersion('1.10.0') else object


class MaintenanceModeMiddleware(_base):
    def process_request(self, request):
        """
        Check if the current site is in maintenance.
        """

        # First check things that don't require a database access:

        # Allow access if remote ip is in INTERNAL_IPS
        if request.META.get('REMOTE_ADDR') in settings.INTERNAL_IPS:
            return None

        # Check if the staff the user is allowed
        if hasattr(request, 'user'):
            if request.user.is_superuser:
                return None

            if not MAINTENANCE_BLOCK_STAFF and request.user.is_staff:
                return None

        # ok let's look at the db
        site = Site.objects.get_current()
        try:
            maintenance = Maintenance.objects.get(site=site)
        except Maintenance.DoesNotExist:
            # Allow access if no matching Maintenance object exists
            return None

        # Allow access if maintenance is not being performed
        if not maintenance.is_being_performed:
            return None

        # Check if a path is explicitly excluded from maintenance mode
        ignored_url_list = set(
            maintenance.ignored_url_patterns() + MAINTENANCE_ADMIN_IGNORED_URLS
        )

        ignored_url_patterns = tuple(
            re.compile(r'{}'.format(url)) for url in ignored_url_list
        )

        request_path = request.path_info.lstrip("/")

        for url in ignored_url_patterns:
            if url.match(request_path):
                return None

        # Otherwise show the user the 503 page
        resolver = resolvers.get_resolver(None)

        resolve = resolver.resolve_error_handler
        callback, param_dict = resolve('503')
        return callback(request, **param_dict)



VERSION = (1, 1, 10)  # following PEP 386
DEV_N = None


def get_version():
    version = '{0}.{1}'.format(VERSION[0], VERSION[1])
    if VERSION[2]:
        version = '{0}.{1}'.format(version, VERSION[2])
    try:
        if VERSION[3]:
            version = '{0}.{1}'.format(version, VERSION[3])
    except IndexError:
        pass
    return version


__version__ = get_version()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='IgnoredURL',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('pattern', models.CharField(max_length=255)),
                ('description', models.CharField(help_text=b'What this URL pattern covers.', max_length=75)),
            ],
        ),
        migrations.CreateModel(
            name='Maintenance',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_being_performed', models.BooleanField(default=False, verbose_name=b'In Maintenance Mode')),
                ('site', models.ForeignKey(to='sites.Site', on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name': 'Maintenance Mode',
                'verbose_name_plural': 'Maintenance Mode',
            },
        ),
        migrations.AddField(
            model_name='ignoredurl',
            name='maintenance',
            field=models.ForeignKey(to='maintenancemode.Maintenance', on_delete=models.CASCADE),
        ),
    ]



from django.contrib import admin
from django.contrib.sites.models import Site

from maintenancemode.models import Maintenance, IgnoredURL, populate


class IgnoredURLInline(admin.TabularInline):
    model = IgnoredURL
    extra = 3


class MaintenanceAdmin(admin.ModelAdmin):
    inlines = [IgnoredURLInline, ]
    list_display = ['__str__', 'is_being_performed']
    readonly_fields = ('site',)
    actions = None

    def get_queryset(self, request):
        # creates missing maintenances if necessary
        populate()
        return super(MaintenanceAdmin, self).get_queryset(request)

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False


admin.site.register(Maintenance, MaintenanceAdmin)



# -*- coding: utf-8 -*-
from django import VERSION as DJANGO_VERSION
from django.conf import settings
from django.core.management import BaseCommand, CommandError
from django.contrib.sites.models import Site
from maintenancemode.models import Maintenance


class Command(BaseCommand):
    help = (
        "Set maintenance-mode on or off for <site_id>."
        " If no site_id given, will use settings.SITE_ID"
    )

    args = "status [site_id [site_id]*]"

    status_map = {"on": True, "off": False}
    status_rmap = {v: k for k, v in status_map.items()}

    if DJANGO_VERSION >= (1, 8, 0):
        def add_arguments(self, parser):
            parser.add_argument(
                "status",
                choices=("on", "off"),
                help="<status> (on/off)"
            )

            parser.add_argument(
                "args",
                nargs="*",
                type=int,
                help="<site_id>*"
            )

            parser.add_argument(
                "-i", "--ignore-missing",
                action="store_true",
                # type=bool,
                default=False,
                help="ignore any inexistant site_id (default %(default)s)"
            )

    else:
        # pre 1.8 compat using optparse
        # here we have to manually get
        # `status` from `*args` and set it as
        # `options['status']`
        from optparse import make_option

        option_list = BaseCommand.option_list + (
            make_option(
                "-i", "--ignore-missing",
                action="store_true",
                # type=bool,
                default=False,
                help="ignore any inexistant site_id (default False)"
            )
        )

    def _compat_parse_args(self, args, options):
        if DJANGO_VERSION < (1, 8, 0):
            try:
                status, args = args[0], args[1:]
            except IndexError:
                raise CommandError("missing <status> (on / off) argument")
            if status not in self.status_map:
                raise CommandError(
                    "invalid <status> argument '{}'"
                    " (should be either 'on' or 'off')".format(status)
                )
            options["status"] = status

        return args, options

    def handle(self, *args, **options):
        args, options = self._compat_parse_args(args, options)

        # print "args : {}".format(args)
        # print "options : {}".format(options)

        site_ids = args or (settings.SITE_ID, )
        known_sites = Site.objects.all()
        missings = \
            set(site_ids) - set(known_sites.values_list("id", flat=True))

        if missings and not options["ignore_missing"]:
            raise CommandError(
                "Unknown site ids: {}".format(" ".join(map(str, missings)))
            )

        status = self.status_map[options["status"]]
        for site_id in site_ids:
            # handle the case of --ignore-missing
            if site_id in missings:
                if options["verbosity"] >= 1:
                    self.stderr.write(
                        "unknown site {} - skipping\n".format(site_id)
                    )
                    continue

            m, created = Maintenance.objects.get_or_create(site_id=site_id)

            if m.is_being_performed == status:
                if options["verbosity"] >= 1:
                    self.stderr.write(
                        "Site {site_id} ({site}) already {status}\n".format(
                            site_id=site_id,
                            site=m.site,
                            status=self.status_rmap[status]
                        )
                    )
                continue

            m.is_being_performed = status
            m.save()
            if options["verbosity"] >= 1:
                self.stderr.write(
                    "Site {site_id} ({site}) is now {status}\n".format(
                        site_id=site_id,
                        site=m.site,
                        status=self.status_rmap[status]
                    )
                )



from inspect import getmembers

from django import get_version
from django.conf import settings

from distutils.version import StrictVersion
DJANGO_VERSION = StrictVersion(get_version())
MAINTENANCE_503_TEMPLATE = getattr(settings,
                                   'MAINTENANCE_503_TEMPLATE',
                                   '503.html')
MAINTENANCE_ADMIN_IGNORED_URLS = getattr(settings,
                                         'MAINTENANCE_ADMIN_IGNORED_URLS',
                                         ['^admin'])
MAINTENANCE_BLOCK_STAFF = getattr(settings, 'MAINTENANCE_BLOCK_STAFF', False)


class AppSettings(object):
    """
    An app setting object to be used for handling app setting defaults
    gracefully and providing a nice API for them. Say you have an app
    called ``myapp`` and want to define a few defaults, and refer to the
    defaults easily in the apps code. Add a ``settings.py`` to your app::

        from path.to.utils import AppSettings

        class MyAppSettings(AppSettings):
            SETTING_1 = "one"
            SETTING_2 = (
                "two",
            )

    Then initialize the setting with the correct prefix in the location of
    of your choice, e.g. ``conf.py`` of the app module::

        settings = MyAppSettings(prefix="MYAPP")

    The ``MyAppSettings`` instance will automatically look at Django's
    global setting to determine each of the settings and respect the
    provided ``prefix``. E.g. adding this to your site's ``settings.py``
    will set the ``SETTING_1`` setting accordingly::

        MYAPP_SETTING_1 = "uno"

    Usage
    -----

    Instead of using ``from django.conf import settings`` as you would
    usually do, you can switch to using your apps own settings module
    to access the app settings::

        from myapp.conf import settings

        print myapp_settings.MYAPP_SETTING_1

    ``AppSettings`` instances also work as pass-throughs for other
    global settings that aren't related to the app. For example the
    following code is perfectly valid::

        from myapp.conf import settings

        if "myapp" in settings.INSTALLED_APPS:
            print "yay, myapp is installed!"

    Custom handling
    ---------------

    Each of the settings can be individually configured with callbacks.
    For example, in case a value of a setting depends on other settings
    or other dependencies. The following example sets one setting to a
    different value depending on a global setting::

        from django.conf import settings

        class MyCustomAppSettings(AppSettings):
            ENABLED = True

            def configure_enabled(self, value):
                return value and not self.DEBUG

        custom_settings = MyCustomAppSettings("MYAPP")

    The value of ``custom_settings.MYAPP_ENABLED`` will vary depending on the
    value of the global ``DEBUG`` setting.

    Each of the app settings can be customized by providing
    a method ``configure_<lower_setting_name>`` that takes the default
    value as defined in the class attributes as the only parameter.
    The method needs to return the value to be use for the setting in
    question.
    """
    def __dir__(self):
        return sorted(list(set(self.__dict__.keys() + dir(settings))))

    __members__ = lambda self: self.__dir__()

    def __getattr__(self, name):
        if name.startswith(self._prefix):
            raise AttributeError(
                "{0} object has no attribute {1}".format(
                    (self.__class__.__name__, name)
                )
            )
        return getattr(settings, name)

    def __setattr__(self, name, value):
        super(AppSettings, self).__setattr__(name, value)
        if name in dir(settings):
            setattr(settings, name, value)

    def __init__(self, prefix):
        super(AppSettings, self).__setattr__('_prefix', prefix)
        for setting, class_value in getmembers(self.__class__):
            if setting == setting.upper():
                prefixed = "{0}_{1}".format(prefix.upper(), setting.upper())
                configured_value = getattr(settings, prefixed, class_value)
                callback_name = "configure_{}".format(setting.lower())
                callback = getattr(self, callback_name, None)
                if callable(callback):
                    configured_value = callback(configured_value)
                delattr(self.__class__, setting)
                setattr(self, prefixed, configured_value)



import json
from django.template import loader, RequestContext

from django import VERSION as DJANGO_VERSION
from maintenancemode.http import HttpResponseTemporaryUnavailable
from maintenancemode.utils.settings import MAINTENANCE_503_TEMPLATE


def temporary_unavailable(request, template_name=MAINTENANCE_503_TEMPLATE):
    """
    Default 503 handler
    """
    # let's be kind to json api calls...
    # NB we specify that we're down for maintenance 
    # so a plain 
    if request.META.get("CONTENT_TYPE") == "application/json":
        content = json.dumps({
                "code": 503,
                "error": "temporarily_unavailable",
                "reason": "maintenance",
                "error_description": "Sorry, the service is temporarily down for maintenance"
                })
        content_type = "application/json"
    else:
        args = [template_name, {'request_path': request.path}]
        if DJANGO_VERSION < (1, 10, 0):
            args.append(RequestContext(request))
        content = loader.render_to_string(*args)
        content_type = "text/html"

    return HttpResponseTemporaryUnavailable(content, content_type=content_type)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-09-20 12:55
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('maintenancemode', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='maintenance',
            name='site',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='sites.Site'),
        ),
    ]



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.db import IntegrityError
from django.contrib.sites.models import Site


@python_2_unicode_compatible
class Maintenance(models.Model):
    site = models.OneToOneField(Site, on_delete=models.CASCADE)
    is_being_performed = models.BooleanField(
        _('In Maintenance Mode'), default=False
    )

    class Meta:
        verbose_name = verbose_name_plural = _('Maintenance Mode')

    def __str__(self):
        return self.site.domain

    def ignored_url_patterns(self):
        qs = self.ignoredurl_set.values_list(
            "pattern", flat=True
            )
        return list(qs)


@python_2_unicode_compatible
class IgnoredURL(models.Model):
    maintenance = models.ForeignKey(Maintenance, on_delete=models.CASCADE)
    pattern = models.CharField(max_length=255)
    description = models.CharField(
        max_length=75, help_text=_('What this URL pattern covers.')
    )

    def __str__(self):
        return self.pattern


def populate():
    """
    creates Maintenance objects for all sites (if necessary)
    """
    for site in Site.objects.all():
        try:
            Maintenance.objects.get_or_create(site=site)
        except IntegrityError as e:
            # MySQL can be a bit annoying sometimes
            pass




from setuptools import setup, find_packages

try:
    README = open('README.rst').read()
except Exception:
    README = None

setup(
    author='Brandon Taylor',
    author_email='alsoicode@gmail.com',
    classifiers=['Development Status :: 5 - Production/Stable',
                 'Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Apache Software License',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python',
                 'Topic :: Utilities'],
    description='Database-driven way to put your Django site into maintenance mode.',
    include_package_data=True,
    install_requires=['django'],
    license='APL',
    long_description=README,
    name='django-maintenancemode-2',
    packages=find_packages(exclude=['testproject']),
    url='https://github.com/alsoicode/django-maintenancemode-2',
    version=__import__('maintenancemode').__version__,
    zip_safe=False
)
